/*
	Noise.h
	Has all the Perlin noise routines
	
	Note: I use a recursive interpolation function to get the 4-D noise and a non-recursive
	function to get the 2-D noise.  The reason for this is simply that while the recursive function
	would work fine with 2-D vectors with some modifications, I was getting sick of debugging recursive
	functions to make sure everything worked properly.  The recursive interpolation function should work for
	N-dimensional vectors (I think) with some fairly minor modifications.
*/

#define RAND(x)	white_noise[int(x) & (RAND_SIZE-1)]

#define NOISE(x,y,z,w) RAND(((x+y)*z)+w) //Hash routine for x dir of balloons
#define NOISE2(x,y) RAND((x*NOISE_SIZE)+y)  //Hash routine for clouds
#define NOISE3(x,y,z,w) RAND(((w+z)*x)+y) //Hash routine for y dir of balloons

float white_noise[RAND_SIZE]; //Random noise that perlin noise runs off of

void init_noise()
{ 
	srand(timeGetTime());
	for (int x = 0; x < RAND_SIZE; x++)
	{
		white_noise[x] = (float)rand()/(RAND_MAX);
	}
}


float linear_noise(float vec[4], int dim) //Recursive Perlin noise function
{
	if (dim == 0)
	{
		int iv = vec[0];
		float v = vec[0], dv;

		dv = v - iv;

		return (NOISE(vec[0],vec[1],vec[2],vec[3]) + dv*(NOISE(vec[0]+1,vec[1],vec[2],vec[3])-NOISE(vec[0],vec[1],vec[2],vec[3])));
	}

	int iv = vec[dim];
	float v = vec[dim], dv, bot, top;

	dv = v - iv;

	bot = linear_noise(vec, dim-1);
	vec[dim]++;
	top = linear_noise(vec, dim-1);

	return (bot + dv*(top-bot));
}

float linear_noise2(float vec[4], int dim) //Recursive function that uses second hash macro
{
	if (dim == 0)
	{
		int iv = vec[0];
		float v = vec[0], dv;

		dv = v - iv;

		return (NOISE3(vec[0],vec[1],vec[2],vec[3]) + dv*(NOISE3(vec[0]+1,vec[1],vec[2],vec[3])-NOISE3(vec[0],vec[1],vec[2],vec[3])));
	}

	int iv = vec[dim];
	float v = vec[dim], dv, bot, top;

	dv = v - iv;

	bot = linear_noise2(vec, dim-1);
	vec[dim]++;
	top = linear_noise2(vec, dim-1);

	return (bot + dv*(top-bot));
}
	
float turbulence(float x, float y, float z, float w, bool dir)
{
	float t = 0, size = pixel_size, scale = 1, check;
	float vec[4];
	
	vec[0] = x;
	vec[1] = y;
	vec[2] = z;
	vec[3] = w;
	
	while (scale > pixel_size)
	{
		if (dir) check = linear_noise(vec,3);
		else check = linear_noise2(vec, 3);

		t += check * size;
		size *= 2;
		scale /= 2;
	
		for (int i = 0; i < 4; i++)
			vec[i] /= 2;
	}

	return(t);
}

float linear_noise(float u, float v) //Non-recursive Perlin noise function
{
	int iu = u, iv = v, ip, iq;
	float du, dv, bot, top;

	du = u - iu;
	dv = v - iv;

	iu = iu % NOISE_SIZE;
	iv = iv % NOISE_SIZE;

	ip = (iu + 1) % NOISE_SIZE;
	iq = (iv + 1) % NOISE_SIZE;

	bot = NOISE2(iu,iv) + du*(NOISE2(ip,iv) - NOISE2(iu,iv));
	top = NOISE2(iu,iq) + du*(NOISE2(ip,iq) - NOISE2(iu,iq));

	return (bot + dv*(top-bot));
}


float turbulence(float u, float v)
{
	float t = 0, size = pixel_size, scale = 1;

	while (scale > pixel_size)
	{
		t += linear_noise((u*scale), (v*scale))*size;
		scale /= 2;
		size *= 2;
	}
	return(t);
}
