/*
	sky.h
	Renders sky-dome and creates cloud texture
*/

GLuint texture;

//Render sky uses the vector "high" as the definition of the peak of the dome. Every other vertex is high
//rotated by a certain amount around the x and z axes
void render_sky(int detail) //detail should be an odd number
{
	float ang_step = angle/detail;
	int x,y,i,j;
	TMatrix rot_x, rot_y, step_x = rotateZ(ang_step);

	Spline *sky;
	sky = new Spline[detail];

	for (x = 0; x < detail; x++)
		sky[x] = Spline(detail);

	glColor3f(1,1,1);
	Vector temp;
	for (x = -detail/2, i = 0; x <= detail/2 ; x++, i++)
		for (y = -detail/2, j++; y <= detail/2; y++, j++)
		{
			temp = high;
			rot_x = rotateZ(ang_step*x);
			rot_y = rotateX(ang_step*y);
			temp = rot_x * temp;
			temp = rot_y * temp;
			
			sky[x+detail/2][y+detail/2] = temp;
		}
	
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	
	glBindTexture(GL_TEXTURE_2D, texture);
	int t = 0;
	float u, ui, v;
	for (x = 0; x < detail-1; x++)
	{
		glBegin(GL_TRIANGLE_STRIP);
		for (y = 0; y < detail; y++)
		{
			u = (float)x/(detail-1);
			ui = (float)(x+1)/(detail-1);
			v = (float)y/(detail-1);

			glTexCoord2f(u,v);
			glVertex3f(sky[x][y].x, sky[x][y].y, sky[x][y].z);
			glTexCoord2f(ui,v);
			glVertex3f(sky[x+1][y].x, sky[x+1][y].y, sky[x+1][y].z);


		}
		glEnd();
	}

	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	free(sky);
}

#define ABS(x) (x < 0 ? -x : x)
float cutoff = .5;
float clouds[CLOUD_SIZE][CLOUD_SIZE][3];

//Uses Perlin noise to get initial values, then runs it through an exponential function to make more
//dramatically white clouds, and fades the clouds out if it's near the edge of the dome (otherwise, there
//will be an abrupt change from white to pure blue which looks pretty ugly
void gen_clouds()
{
	float value, dist; 

	for (int i = 0; i < CLOUD_SIZE; i++)
	{
		for (int j = 0; j < CLOUD_SIZE; j++)
		{
			value = turbulence(i,j);
			dist = MAX(fabs(i-CLOUD_SIZE/2), fabs(j-CLOUD_SIZE/2))/(CLOUD_SIZE/2);	
			value -= .05/(value*value*value) + .2;
			if (dist > cutoff) value = value - value*((dist-(1-cutoff))/cutoff); 
			if (value < 0) value = 0;

			clouds[i][j][0] = .2 + value*.8;
			clouds[i][j][1] = .5 + value*.5;
			clouds[i][j][2] = 1;

		}
	}
		
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, CLOUD_SIZE, CLOUD_SIZE, 0, GL_RGB, GL_FLOAT, clouds);
}
