/*
	String.h
	Models the behavior of the string for each balloon

	Note: The method that is used in this program is a flawed one. The distance between each node is not
	governed by STRING_LENGTH, but rather by the elasticity and gravity defined, as well as the velocity of
	the balloon tugging on the string.  Depending on these factors, an equilibrium will be reached after a bit
	of growth by the string. I have defined STRING_LENGTH as 5 because it's fairly near the equilibrium for the
	various defined factors and therefore the strings don't become about 100x longer than they were meant to be.
	If you want to see for yourself, define STRING_LENGTH as .5 and watch them unfurl like the lampreys in
	Half Life.
*/

class String
{
public:
	String(int knots, Vector top);
	String();
	void update(Vector top);
	void render();

private:
	int num_knots;
	Vector *string_pos, *string_vel;
};

String::String() {}

String::String(int knots, Vector top)
{
	string_pos = new Vector[knots];
	string_vel = new Vector[knots];
	num_knots = knots;
	
	string_pos[0] = top;

	for (int x = 1; x < num_knots; x++)
	{
		string_pos[x] = string_pos[x-1];
		string_pos[x].y -= STRING_LENGTH;
	}
}
	
void String::update(Vector top)
{
	string_pos[0] = top;
	Vector up, down, goal;
	float up_length, down_length;

	//For each node along the string, the relative distance of the neighbors and gravity are factored
	//into the node's velocity, which is dampened and then added to the position of the node
	for (int x = 1; x < num_knots-1; x++)
	{
		up = string_pos[x-1]-string_pos[x];
		down = string_pos[x+1]-string_pos[x];
		up_length = length_squared(up);
		down_length = length_squared(down);
		
		string_vel[x] += up*((up_length-L_SQUARED)*elast);
		string_vel[x] += down*((down_length-L_SQUARED)*elast);
		
		string_vel[x] += grav;

		string_vel[x] = string_vel[x]*.99; 
		
		string_vel[x].y = MIN(string_vel[x].y, -grav.y); //Without these limits, the string will bounce up
		string_vel[x].y = MAX(string_vel[x].y, grav.y);  //and down like crazy
	}

	up = string_pos[num_knots-2]-string_pos[num_knots-1];
	up_length = length_squared(up);
	string_vel[num_knots-1] += up*((up_length-L_SQUARED)*elast);

	string_vel[num_knots-1] = string_vel[num_knots-1]*.99;
	string_vel[num_knots-1] += grav;

	string_vel[num_knots-1].y = MIN(string_vel[num_knots-1].y, -grav.y);
	string_vel[num_knots-1].y = MAX(string_vel[num_knots-1].y, grav.y);

	for (int y = 1; y < num_knots; y++)
		string_pos[y] += string_vel[y];
}

		
void String::render()
{
	glBegin(GL_LINE_STRIP);
	for (int x = 0; x < num_knots; x++)
	{
		glVertex3d(string_pos[x].x-1, string_pos[x].y, string_pos[x].z);
	}
	glEnd();
}	
