#ifndef _TMATRIX_H
#define _TMATRIX_H

#include "vector.h"
#include <stdlib.h>


#define pi 3.1415926535897932384626433832795

class TCol //4D Vector for use in matrices
{
public:
	double &operator[] (int k);
	double values[4];

private:
};

double &TCol::operator [] (int k)
{
	if (k > 4 || k < 0) exit(-1);  //Someday to be replaced with error log

	return values[k];
}


class TMatrix
{
public:
	/////////////////////////////////////
	//Constructors
	TMatrix();
	
	/////////////////////////////////////
	//Transformations
	TMatrix scaleXYZ(double x, double y, double z);

	
	//Apply matrix to point, vector, or tri-mesh
	Vector transform(Vector &point); 
	//Trimesh transform(Trimesh mesh);
	
	//////////////////////////////////////
	//Operators
	TCol &operator[] (int k);
	
	TMatrix operator* (TMatrix rhs) const; //Essentially matrix concatenation
	TMatrix operator* (int rhs) const;	   //Matrix scaling (all elements times k)
	TMatrix operator+ (TMatrix rhs) const;
	TMatrix operator- (TMatrix rhs) const;
	

	void operator *= (TMatrix rhs);
	void operator *= (int rhs);
	void operator += (TMatrix rhs);
	void operator -= (TMatrix rhs);

	Vector operator* (Vector rhs);

private:
	TCol matrix[4];
};


TCol &TMatrix::operator[] (int k)	//would be used in conjunction with TCol to allow for
{									//matrix[x][y] style access			
	if (k >= 4 || k < 0) exit(-1); 

	return matrix[k];
}


///////////////////////////////////////////////////////////////////////////////////

TMatrix::TMatrix()
{
	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			matrix[x][y] = 0.0f;

	for (x = 0; x < 4; x++) //Create identity matrix
		matrix[x][x] = 1.0f;
}


////////////////////////////////////////////////////////////////////////////////////
TMatrix TMatrix::operator+ (TMatrix rhs) const
{
	TMatrix temp;

	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			temp[x][y] = matrix[x].values[y] + rhs[x][y];

	return temp;
}

TMatrix TMatrix::operator- (TMatrix rhs) const
{
	TMatrix temp;

	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			temp[x][y] = matrix[x].values[y] - rhs[x][y];

	return temp;
}

TMatrix TMatrix::operator* (TMatrix rhs) const
{
	TMatrix temp;

	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			for (int z = 0; z < 4; z++)
				temp[x][y] += matrix[z].values[y] * rhs[x][z];
	
	return temp;
}

TMatrix TMatrix::operator* (int rhs) const
{
	TMatrix temp;
	
	for (int x = 0; x < 4; x++)
		for (int y = 0; y < 4; y++)
			temp[x][y] *= rhs;

	return temp;
}


void TMatrix::operator += (TMatrix rhs)
{
	(*this) = ((*this) + rhs);
}

void TMatrix::operator -= (TMatrix rhs)
{
	(*this) = ((*this) - rhs);
}

void TMatrix::operator *= (TMatrix rhs)
{
	(*this) = ((*this) * rhs);
}

void TMatrix::operator *= (int rhs)
{
	(*this) = ((*this) * rhs);
}

Vector TMatrix::operator* (Vector rhs) //Applying a matrix to a vector
{
	Vector temp;
	
	temp.x =  rhs.x * matrix[0].values[0] +
			  rhs.y * matrix[1].values[0] +
			  rhs.z * matrix[2].values[0] +
			  matrix[3].values[0];
	temp.y =  rhs.x * matrix[0].values[1] +
			  rhs.y * matrix[1].values[1] +
			  rhs.z * matrix[2].values[1] +
			  matrix[3].values[1];
	temp.z =  rhs.x * matrix[0].values[2] +
			  rhs.y * matrix[1].values[2] +
			  rhs.z * matrix[2].values[2] +
			  matrix[3].values[2];

	if (rhs.is_norm()) temp = temp.normal();

	return temp;
}



Vector TMatrix::transform(Vector &point)
{	
	Vector temp;
	
	temp.x =  point.x * matrix[0][0] +
			  point.y * matrix[1][0] +
			  point.z * matrix[2][0] +
			  matrix[3][0];
	temp.y =  point.x * matrix[0][1] +
			  point.y * matrix[1][1] +
			  point.z * matrix[2][1] +
			  matrix[3][1];
	temp.z =  point.x * matrix[0][2] +
			  point.y * matrix[1][2] +
			  point.z * matrix[2][2] +
			  matrix[3][2];

	
	  if (point.is_norm()) temp = temp.normal();
	point = temp;

	return point;
}


////////////////////////////////////////////////////////////////////////////////

TMatrix rotateX(double theta)
{
	TMatrix temp;
	theta /= (double)180/pi;

	temp[1][1] = cosf(theta);
	temp[2][1] = -sinf(theta);
	temp[1][2] = sinf(theta);
	temp[2][2] = cosf(theta);

	return temp;
}

TMatrix rotateY(double theta)
{
	TMatrix temp;
	theta /= (double)180/pi;

	temp[0][0] = cosf(theta);
	temp[2][0] = sinf(theta);
	temp[0][2] = -sinf(theta);
	temp[2][2] = cosf(theta);

	return temp;
}

TMatrix rotateZ(double theta)
{
	TMatrix temp;
	theta /= 180/pi;

	temp[0][0] = cos(theta);
	temp[1][0] = -sin(theta);
	temp[0][1] = sin(theta);
	temp[1][1] = cos(theta);
	
	return temp;
}

TMatrix rotateXYZ(double x_axis, double y_axis, double z_axis)
{
	TMatrix temp;
	
	if(x_axis) temp *= rotateX(x_axis);
	if(y_axis) temp *= rotateY(y_axis);
	if(z_axis) temp *= rotateZ(z_axis);

	return temp;
}


TMatrix translate(double x, double y, double z)
{
	TMatrix temp;

	temp[3][0] = x;
	temp[3][1] = y;
	temp[3][2] = z;

	return temp;
}

TMatrix scale(double size)
{
	TMatrix temp;

	for (int x = 0; x < 3; x++)
		temp[x][x] = size;

	return temp;
}


void transform(float *mat, Vector &vec)
{
	Vector result;
	result.x = (mat[0] * vec.x) +
			   (mat[4] * vec.y) +	
			   (mat[8] * vec.z) +
			   mat[12];
	result.y = (mat[1] * vec.x) +
			   (mat[5] * vec.y) +	
			   (mat[9] * vec.z) +
			   mat[13];
	result.z = (mat[2] * vec.x) +
			   (mat[6] * vec.y) +	
			   (mat[10] * vec.z) +
			   mat[14];

	vec.x = result.x;
	vec.y = result.y;
	vec.z = result.z;
}




#endif        